# Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.

from cmf.monitor import schema

class ParseError(Exception):
  pass

def get_leaf_dict(dic, *path):
  """
  Given a Hadoop metrics json dict, return the leaf metrics dictionary
  specified by path. May raise ParseError.

  Returns a DictWithAttr.
  """
  traversed = []
  curr = dic

  for component in path:
    traversed.append(component)
    curr = curr.get(component)
    if curr is None:
      err = "Cannot find component %s in json" % ('->'.join(traversed),)
      raise ParseError(err)

  #
  # `curr' should be a list of size 1, and its only element is
  # another list of:
  #   { context dict }
  #   { real metrics dict }
  #
  # `curr' may also be an empty list iff the service doesn't not public metrics
  #
  if not isinstance(curr, list):
    err = "Expecting a list at %s. Got %r" % ('->'.join(traversed), curr)
    raise ParseError(err)

  if len(curr) == 0:
    return DictWithAttr({})
  elif len(curr) != 1:
    err = "Expecting only one array element in %s. Got %r" % \
        ('->'.join(traversed), curr)
    raise ParseError(err)

  curr = curr[0]
  if len(curr) != 2:
    err = "Expecting two elements (context and metrics) in %s. Got %r" % \
        ('->'.join(traversed), curr)
    raise ParseError(err)

  return DictWithAttr(curr[1])

def get_beans(dic, names):
  """
  Given a Hadoop or Cloudera Manager JMX json dict, returns a map of beans found
  matching the input set of name patterns as a dictionary from name pattern -> bean
  as a dictionary. May raise ParseError.

  Returns a DictWithAttr.
  """
  beans = dic.get('beans')
  if beans is None:
    err = "Cannot find beans list in input json"
    raise ParseError(err)

  if not isinstance(beans, list):
    err = "Expecting a beans list. Got %r" % (beans, )
    raise ParseError(err)
  
  ret = {}
  for bean in beans:
    d = DictWithAttr(bean)
    name = d.get('name')
    for pattern in names:
      if pattern.match(name):
        ret[pattern] = d
        break
    if len(ret.keys()) == len(names):
      break

  return ret


class DictWithAttr(dict):
  """
  A wrapper class to allow attribute access for dictionary-like
  objects (e.g. json objects).
  """
  def __init__(self, dictionary, raise_when_not_found=False):
    try:
      dict.__init__(self, dictionary)
    except:
      raise ParseError("Cannot initialize dictionary with %r" % (dictionary,))
    self._raise = raise_when_not_found

  def __getattr__(self, attr):
    try:
      res = self[attr]
      if isinstance(res, dict):
        # Recursively wrap the result if it's also a dictionary
        return DictWithAttr(res)
      return res
    except KeyError:
      if self._raise:
        raise AttributeError(attr)
      else:
        return None

_JVM_METRIC_NAMES = \
  {
  'gcCount' : 'gcCount', 
  'gcTimeMillis' : 'gcTimeMillis',
  'memHeapUsedM' : 'memHeapUsedM',
  'memHeapCommittedM' : 'memHeapCommittedM',
  'threadsBlocked' : 'threadsBlocked',
  'threadsNew' : 'threadsNew',
  'threadsRunnable' : 'threadsRunnable',
  'threadsTerminated' : 'threadsTerminated',
  'threadsTimedWaiting' : 'threadsTimedWaiting',
  'threadsWaiting' : 'threadsWaiting'
  }

_JVM_JMX_METRIC_NAMES = \
  {
  'gcCount' : 'GcCount', 
  'gcTimeMillis' : 'GcTimeMillis',
  'memHeapUsedM' : 'MemHeapUsedM',
  'memHeapCommittedM' : 'MemHeapCommittedM',
  'threadsBlocked' : 'ThreadsBlocked',
  'threadsNew' : 'ThreadsNew',
  'threadsRunnable' : 'ThreadsRunnable',
  'threadsTerminated' : 'ThreadsTerminated',
  'threadsTimedWaiting' : 'ThreadsTimedWaiting',
  'threadsWaiting' : 'ThreadsWaiting'
  }

_BYTES_PER_MB = 1048576

def add_jvm_metrics(update, metrics, jmx=False):
  if jmx:
    jvm_present = (metrics.jvm is not None)
    names = _JVM_JMX_METRIC_NAMES
  else:
    jvm_present = (len(metrics.jvm) != 0)
    names = _JVM_METRIC_NAMES

  if jvm_present:
    update.add_metric(schema.JVM_GC_COUNT,
                      metrics.jvm.get(names['gcCount']))
    update.add_metric(schema.JVM_GC_TIME_MS,
                      metrics.jvm.get(names['gcTimeMillis']))
    update.add_metric(schema.JVM_HEAP_USED_MB,
                      metrics.jvm.get(names['memHeapUsedM']))
    update.add_metric(schema.JVM_HEAP_COMMITTED_MB,
                      metrics.jvm.get(names['memHeapCommittedM']))
    totalThreads = 0

    threadsBlocked = metrics.jvm.get(names['threadsBlocked'])
    update.add_metric(schema.JVM_BLOCKED_THREADS, threadsBlocked)
    totalThreads += threadsBlocked

    threadsNew = metrics.jvm.get(names['threadsNew'])
    update.add_metric(schema.JVM_NEW_THREADS, threadsNew)
    totalThreads += threadsNew

    threadsRunnable = metrics.jvm.get(names['threadsRunnable'])
    update.add_metric(schema.JVM_RUNNABLE_THREADS, threadsRunnable)
    totalThreads += threadsRunnable

    threadsTerminated = metrics.jvm.get(names['threadsTerminated'])
    update.add_metric(schema.JVM_TERMINATED_THREADS, threadsTerminated)
    totalThreads += threadsTerminated

    threadsTimedWaiting = metrics.jvm.get(names['threadsTimedWaiting'])
    update.add_metric(schema.JVM_TIMED_WAITING_THREADS, threadsTimedWaiting)
    totalThreads += threadsTimedWaiting

    threadsWaiting = metrics.jvm.get(names['threadsWaiting'])
    update.add_metric(schema.JVM_WAITING_THREADS, threadsWaiting)
    totalThreads += threadsWaiting

    update.add_metric(schema.JVM_TOTAL_THREADS, totalThreads)

  # The max memory metric is not exposed over the hadoop metrics jvmx bean, so
  # we get it instead from the java_memory bean. See OPSAPS-5750.
  if jmx:
    if metrics.java_memory is not None:
      max_memory_bytes = metrics.java_memory.HeapMemoryUsage.max
      update.add_metric(schema.JVM_MAX_MEMORY_MB,
                        max_memory_bytes / float(_BYTES_PER_MB))
  else:
    if jvm_present:
      update.add_metric(schema.JVM_MAX_MEMORY_MB,
                        metrics.jvm.maxMemoryM)
