# Copyright (c) 2011 Cloudera, Inc. All rights reserved.

import logging
import os
import re
import resource
import shutil
import simplejson
import tempfile
import threading
import time
import base64
from nose.plugins.skip import SkipTest

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

import avro.io
import avro.protocol

import cmf.monitor.namenode.json_parser

from cmf.monitor import conf, jmx_beans
from cmf.monitor.count_down_latch import CountDownLatch
from cmf.monitor.fs_update import FsUpdate
from cmf.monitor.nic_update import NicUpdate
from cmf.monitor.host_update import HostUpdate
from cmf.monitor.host_dir_update import HostDirUpdate
from cmf.monitor.json_metrics_utils import DictWithAttr, get_leaf_dict, \
   get_beans
from cmf.monitor.json_metrics_utils import ParseError
from cmf.monitor.monitor_properties import ConfParser
from cmf.monitor.proc_metrics_utils import get_fd_count, get_fd_max
from cmf.monitor.proc_metrics_utils import get_swap_out
from cmf.monitor.stoppable_thread import StoppableThread
from cmf.monitor.wakeable_thread import WakeableThread
from cmf.monitor.disk_update import DiskUpdate

from nose.tools import assert_equals, assert_not_equals, assert_raises
from nose.tools import assert_false, assert_true


LOG = logging.getLogger(__name__)

def make_config_file(content):
  """
  Creates a cloudera-monitor.properties file in a tmp directory,
  and return the absolute path to that directory.
  """
  conf_dir = tempfile.mkdtemp(suffix='.scm-agent-test')
  try:
    path = os.path.join(conf_dir, conf.CLOUDERA_MONITOR_PROPERTIES)
    file(path, 'w').write(content)
    return conf_dir
  except:
    shutil.rmtree(conf_dir)
    raise

def test_dict_with_attr():
  raw = dict(foo=1, bar=dict(baz=2))

  d = DictWithAttr(raw)
  assert_equals(None, d.hello, "Returns None instead of raising")
  assert_not_equals(None, d.bar)

  # d.bar itself should support getattr
  assert_not_equals(None, d.bar.baz)


def test_get_leaf_dict():
  raw = simplejson.loads(cmf.monitor.namenode.json_parser.SAMPLE)

  # dfs->FSDirectory is an empty leaf
  res = get_leaf_dict(raw, 'dfs', 'FSDirectory')
  assert_equals(0, len(res))

  # jvm->metrics is a non-empty leaf
  res = get_leaf_dict(raw, 'jvm', 'metrics')
  assert_equals(3, res.gcCount)

  # jvm is not a leaf
  assert_raises(ParseError, get_leaf_dict, raw, 'jvm')

  assert_raises(ParseError, get_leaf_dict, raw, 'blah')


def test_get_beans():
  raw = simplejson.loads(cmf.monitor.servicemonitor.json_parser.SAMPLE)

  beans = get_beans(raw, set([re.compile('does not exist')]))
  assert_equals(0, len(beans.keys()))

  test_pattern = re.compile('java.lang:type=MemoryPool,name=CMS Old Gen')

  beans = get_beans(raw, set([jmx_beans.EVENT_STATS, test_pattern]))
  assert_equals(5, beans[jmx_beans.EVENT_STATS].SentCount)
  assert_true(beans[test_pattern] is not None)

def test_conf_parse():
  text = """[section_a]
uk = england,ireland,scotland
china = beijing
is_country = true
north_america = 3
"""
  conf_dir = make_config_file(text)
  try:
    cp = ConfParser(conf_dir)
    assert_equals('beijing', cp.get('section_a', 'china'))
    assert_equals(True, cp.getboolean('section_a', 'is_country'))
    assert_equals(3, cp.getint('section_a', 'north_america'))
    assert_equals(['england', 'ireland', 'scotland'],
                  cp.getlist('section_a', 'uk'))
    assert_equals(None, cp.get('section_a', 'NOT THERE'))
  finally:
    shutil.rmtree(conf_dir)


def test_stoppable_thread():
  TIMEOUT = 30
  STOP_TIMEOUT = 5

  class Action(object):
    result = 0
    event = threading.Event()
    # The lock protects `result' and `event'
    lock = threading.Lock()

    @staticmethod
    def reset():
      Action.lock.acquire()
      Action.result = 0
      Action.event.clear()
      Action.lock.release()

    @staticmethod
    def do_sum(first, second):
      LOG.error("do_sum: %r, %s" % (first, second))
      Action.lock.acquire()
      Action.result = first + second
      Action.event.set()
      Action.lock.release()
      raise IOError("Because I want to fail")

  sthread = StoppableThread("Test", 0.1, Action.do_sum, 1, second=2)
  sthread.start()

  # Loop twice to make sure that it's not a one-shot event,
  # despite do_sum() raising an error.
  for _ in range(2):
    # Wait for a summation to happen
    Action.reset()
    Action.event.wait(TIMEOUT)
    # Assuming that Action.result is assigned atomically (which is true for int
    # type), we can read it without locking.
    assert_equals(3, Action.result)

  # It can be stopped properly
  sthread.should_stop()
  time.sleep(STOP_TIMEOUT)
  assert_false(sthread.isAlive())

def test_wakeable_thread():
  TIMEOUT = 2

  class Action(object):
    result = 0

    @staticmethod
    def do_sum(first, second):
      LOG.error("do_sum: %r, %s" % (first, second))
      Action.result = Action.result + first + second
      raise IOError("Because I want to fail")

  wthread1 = WakeableThread("Test", Action.do_sum, 1, second=2)
  wthread1.start()
  wthread2 = WakeableThread("Test", Action.do_sum, 2, second=6)
  wthread2.start()

  # Loop twice to make sure that it's not a one-shot event,
  # despite do_sum() raising an error.
  for _ in range(2):
    Action.result = 0
    latch = CountDownLatch(2)
    wthread1.wake(latch)
    wthread2.wake(latch)
    latch.await()
    assert_equals(11, Action.result)

  # It can be stopped properly
  latch = CountDownLatch(2)
  wthread1.should_stop(latch)
  wthread2.should_stop(latch)
  latch.await()
  time.sleep(TIMEOUT)
  assert_false(wthread1.isAlive())
  assert_false(wthread2.isAlive())


def test_wakeable_thread_timeout():
  TIMEOUT = 2

  class Action(object):
    @staticmethod
    def do(latch):
      latch.countdown()

  latch = CountDownLatch(1)      
  wthread = WakeableThread("Test", Action.do, latch)
  wthread.set_timeout(0.1)
  wthread.start()
  latch.await()

  # It can be stopped properly
  latch = CountDownLatch(1)
  wthread.should_stop(latch)
  latch.await()
  time.sleep(TIMEOUT)
  assert_false(wthread.isAlive())

def test_wakeable_concurrent_wakes():
  TIMEOUT = 2

  class Action(object):
    calls = 0
    # The thread will signal this latch when it's entered its function.
    signal_latch = CountDownLatch(1)
    # The thread will wait for us to signal this latch to finish. This lets us
    # control when we add our second wake call.
    wait_latch = CountDownLatch(1)

    @staticmethod
    def do():
      Action.signal_latch.countdown()
      Action.wait_latch.await()
      Action.calls = Action.calls + 1

  wthread = WakeableThread("Test", Action.do)
  wthread.start()
  latch1 = CountDownLatch(1)
  latch2 = CountDownLatch(1)
  # Wake up the thread.
  wthread.wake(latch1)
  # Wait for it to get into the function. Then reset the latch.
  Action.signal_latch.await()
  Action.signal_latch = CountDownLatch(1)
  # Make a second wake call before letting the thread out of the function.
  wthread.wake(latch2)
  # Let the thread out of the function. We lave the latch signalled so that the
  # thread can make it through both calls.
  assert_equals(0, Action.calls)
  Action.wait_latch.countdown()
  latch1.await()  
  latch2.await()
  assert_equals(2, Action.calls)

  # It can be stopped properly
  latch = CountDownLatch(1)
  wthread.should_stop(latch)
  latch.await()
  time.sleep(TIMEOUT)
  assert_false(wthread.isAlive())

def negative_test_count_down_latch():
  latch = CountDownLatch(1)
  latch.await(0.5)

  latch.countdown()
  assert_raises(Exception, latch.countdown)

def test_fs_update():
  fs_update = FsUpdate('/home', '/dev/sda1').add_metric(metricid=99, value=42)
  avro = fs_update.to_avro_dict()
  assert_equals(avro,
                dict(metrics=[{'id': 99, 'value': 42}],
                     mountpoint='/home',
                     partition='/dev/sda1'))

def test_nic_update():
  nic_update = NicUpdate('eth0').add_metric(metricid=99, value=42)
  avro = nic_update.to_avro_dict()
  assert_equals(avro,
                dict(metrics=[{'id': 99, 'value': 42}],
                     iface='eth0'))

def test_disk_update():
  disk_update = DiskUpdate('/dev/sda1').add_metric(metricid=99, value=42)
  avro = disk_update.to_avro_dict()
  assert_equals(avro,
                dict(metrics=[{'id': 99, 'value': 42}],
                     device='/dev/sda1'))

def test_host_update():
  host_update = HostUpdate("/var/log", "/var/run").\
      add_metric(metricid=1, value=1).\
      add_fs_update(FsUpdate('/vol1', '/dev/sda1').\
                    add_metric(metricid=1, value=1)).\
      add_nic_update(NicUpdate('eth0').
                     add_metric(metricid=1, value=1)).\
      add_disk_update(DiskUpdate('/dev/sda1').
                      add_metric(metricid=1, value=1)).\
      add_dir_update(HostDirUpdate('/dir/1', '/var').
                     add_metric(metricid=2, value=2)).\
      add_dir_update(HostDirUpdate('/dir/2', '/var').
                     add_metric(metricid=2, value=2))

  host_update2 = HostUpdate("/var/log", "/var/run").\
      add_metric(metricid=2, value=2).\
      add_fs_update(FsUpdate('/vol2', '/dev/sda2').
                    add_metric(metricid=2, value=2)).\
      add_nic_update(NicUpdate('eth1').
                     add_metric(metricid=2, value=2)).\
      add_disk_update(DiskUpdate('/dev/sda2').
                      add_metric(metricid=2, value=2)).\
      add_dir_update(HostDirUpdate('/dir/3', '/var').
                     add_metric(metricid=3, value=3)).\
      add_dir_update(HostDirUpdate('/dir/4', '/var').
                     add_metric(metricid=3, value=3))

  # Check that we merge correctly
  host_update.merge(host_update2)
  dic = host_update.to_avro_dict()
  assert_equals(dic['fsUpdates'],
                [ dict(mountpoint='/vol1',
                       metrics=[{'id': 1, 'value': 1}],
                       partition='/dev/sda1'),
                  dict(mountpoint='/vol2',
                       metrics=[{'id': 2, 'value': 2}],
                       partition='/dev/sda2')
                ])
  assert_equals(dic['nicUpdates'],
                [ dict(iface='eth0',
                       metrics=[{'id': 1, 'value': 1}]),
                  dict(iface='eth1',
                       metrics=[{'id': 2, 'value': 2}])
                ])
  assert_equals(dic['diskUpdates'],
                [ dict(device='/dev/sda1',
                       metrics=[{'id': 1, 'value': 1}]),
                  dict(device='/dev/sda2',
                       metrics=[{'id': 2, 'value': 2}])
                ])
  assert_equals(dic['dirUpdates'],
                [ dict(path='/dir/1',
                       mountpoint='/var',
                       metrics=[{'id': 2, 'value': 2}]),
                  dict(path='/dir/2',
                       mountpoint='/var',
                       metrics=[{'id': 2, 'value': 2}]),
                  dict(path='/dir/3',
                       mountpoint='/var',
                       metrics=[{'id': 3, 'value': 3}]),
                  dict(path='/dir/4',
                       mountpoint='/var',
                       metrics=[{'id': 3, 'value': 3}])
                ])
  assert_equals(dic['metrics'], [{'id': 1, 'value': 1},
                                 {'id': 2, 'value': 2}])

def test_proc_metrics():
  pid = os.getpid()
  assert_true(get_fd_count(pid) > 0)
  assert_equals(get_fd_max(pid), resource.getrlimit(resource.RLIMIT_NOFILE)[1])
  swap_out = get_swap_out()
  assert_true(swap_out is not None)
  
  
