// Copyright (c) 2012 Cloudera, Inc.  All rights reserved.
define([
	"cloudera/cmf/include/RepositoryValidator", 
	"cloudera/cmf/include/CDHSelectionPage"
], function(RepositoryValidator, CDHSelectionPage) {
	/**
	 * This module uses KnockoutJS to bind the state of the DOM to the
	 * state of the view model. Either the first step is selected (the cloud credentials page
	 * will be displayed), or the second step is selected (the node specifications page
	 * will be displayed).
	 */
	
	return function(options) {
		var self = this;
		$("#mainForm").validate();

		/**
		 * Initialize the knockout state. Note: we may
		 * be entering the page via the back button.
		 * In which case, we want to initialize the state
		 * based on the DOM value.
		 */
		function initParams() {
			self.chosenStep = ko.observable(-1);
			
			// Image ID is custom
			var isCustomImage = $("input[name=isCustomImage]:checked").val() || "false";
			self.isCustomImage = ko.observable(isCustomImage);
			
			// Private key is supplied by the user
			var isSuppliedKey = $("input[name=isSuppliedKey]:checked").val() || "false";
			self.isSuppliedKey = ko.observable(isSuppliedKey);
			
			var imageId = $("input[name=imageId][type=text]").val() || "";
			self.imageId = ko.observable(imageId);

			var username = $("input[name=username][type=text]").val() || "";
			self.username = ko.observable(username);
			
			var passphrase = $("input[name=passphrase]").val() || "";
			self.passphrase = ko.observable(passphrase);

			var passphraseConfirm = $("input[name=passphraseConfirm]").val() || "";
			self.passphraseConfirm = ko.observable(passphraseConfirm);
			
			var numInstances = $("input[name=numInstances]").val() || "";
			self.numInstances = ko.observable(numInstances);
			
			var customGroup = $("input[name=customGroup]").val() || "";
			self.customGroup = ko.observable(customGroup);

			// No need to show any errors initially.
			self.numInstancesError = ko.observable("");
			self.customGroupError = ko.observable("");
			self.imageIdError = ko.observable("");
			self.usernameError = ko.observable("");
			self.identityError = ko.observable("");
			self.credentialError = ko.observable("");
			self.privateKeyError = ko.observable("");
			self.passphraseError = ko.observable("");
			
			self.selectedRegion = ko.observable($("select[name='region'] option:selected")
				.val() || "");
			self.selectedOs = ko.observable($("select[name='os'] option:selected")
				.val() || "");
			self.selectedOs.subscribe(osCheck);
			osCheck(self.selectedOs());
			self.underlyingImageId = ko.computed(function() {
				return options.amisMap[self.selectedOs()][self.selectedRegion()];
			}, self);
			
			function osCheck(newOs) {
				if (newOs === "ubuntu") {
					// CDH3 doesn't work on Ubuntu 12.04 or later, so if 
					// the user selects Ubuntu 12.04 or later, remove
					// the option of CDH3 from the CDH selection page.
					$("#cdh3Radio").addClass("hidden");
				} else {
					// The user selected something other than Ubuntu 12.04 or 
					// later, so make sure that CDH3 is shown in the CDH selection
					// page.
					$("#cdh3Radio").removeClass("hidden");
				}				
			}
			
			// Observables for the CDH selection page and knockout helper methods
			// are initialized in this call.
			CDHSelectionPage.initializeKnockout(self);
			
			// Enable the submit button by default.
			self.submitted = ko.observable(false);
		}

		initParams();	

		/**
		 * Allow one click on the start installation button.
		 */
		self.enableStartInstall = ko.dependentObservable(function () {
			return !self.submitted();
		}, self);

		function focusAndSelect($el, showError){
			if (showError) {
				$el.closest(".control-group").addClass("error");
			}
			$el.focus();
			$el.select();
		}

		function resetErrors () {
			$(".control-group").removeClass("error");
			$("#numInstancesDiv").removeClass("error");
			$("#customGroupDiv").removeClass("error");
			self.identityError("");
			self.credentialError("");
			self.usernameError("");
			self.imageIdError("");
			self.privateKeyError("");
			self.passphraseError("");
			self.numInstancesError("");
			self.customGroupError("");
		}

		// Reason we don't use knockout to obtain the identity or credential is 
		// that knockout has a bug whereby it doesn't update an observable 
		// when its corresponding DOM element is filled out by the 
		// browser (e.g. due to username/password autocompletion, which is 
		// what occurs here).
		
		function checkEmptyIdentity () {
			if ($("input[name=identity]").val() === '') {
				self.identityError(options.messages.missingIdentity);
				focusAndSelect($("input[name=identity]"), true);
				return false;
			}
			return true;
		}
		
		function checkEmptyCredential () {
			if ($("input[name=credential]").val() === '') {
				self.credentialError(options.messages.missingCredential);
				focusAndSelect($("input[name=credential]"), true);
				return false;
			}
			return true;
		}

		function checkEmptyPrivateKey () {
			if (self.isSuppliedKey() === 'true' && $("[name=privateKey]").val() === '') {
				self.privateKeyError(options.messages.missingPrivateKey);
				focusAndSelect($("input[name=privateKey]"), true);
				return false;
			} else {
				return true;
			}
		}
		
		function checkPassphraseMatch () {
			if (self.isSuppliedKey() === 'true' && self.passphrase() !== 
					self.passphraseConfirm()) {
				self.passphraseError(options.messages.passphraseDontMatch);
				focusAndSelect($("input[name=passphraseConfirm]"), true);
				focusAndSelect($("input[name=passphrase]"), true);
				return false;
			}
			return true;
		}
		
		function checkInvalidNumInstances () {
			var intRegex = /^\d+$/; // to test if the input is an integer
			var parsedNum = parseFloat(self.numInstances());
			if (isNaN(parsedNum) || !intRegex.test(parsedNum) || parsedNum > 
				options.maxNodes || parsedNum <= 0) {
				self.numInstancesError(options.messages.invalidNumInstances);
				$("#numInstancesDiv").addClass("error")
				.find("input[name=numInstances]")
					.focus()
					.select();
				return false;
			}
			
			return true;
		}
		
		// For testing purposes -- so that we can call checkInvalidNumInstances() 
		// from outside this file.
		self.checkInvalidNumInstances = checkInvalidNumInstances;
		
		function checkInvalidCustomGroup () {
			// Starts with a lower-case letter or number. Can contain only 
			// lower-case  letters, numbers, underscores, spaces, or dashes. 
			// Is between 3 to 63 characters in length, inclusive.
			var sgRegex = /^[a-z0-9][a-z0-9_ \-$]{2,62}$/;
			// Since this is an optional parameter, it can be blank.
			if (self.customGroup() !== '' && 
					!sgRegex.test(self.customGroup())) {
				self.customGroupError(options.messages.invalidCustomGroup);
				$("#customGroupDiv").addClass("error")
				.find("input[name=customGroup]")
					.focus()
					.select();
				return false;
			}
			
			return true;
		}
		
		// For testing purposes -- so that we can call checkInvalidCustomGroup() 
		// from outside this file.
		self.checkInvalidCustomGroup = checkInvalidCustomGroup;
		
		function checkEmptyUsername () {
			if (self.isCustomImage() === 'true' && self.username() === '') {
				self.usernameError(options.messages.missingUsername);
				focusAndSelect($("input[name=username]"), true);
				return false;
			} else {
				return true;
			}
		}
		
		function checkEmptyImageId () {
			if (self.isCustomImage() === 'true' && self.imageId() === '') {
				self.imageIdError(options.messages.missingImageId);
				focusAndSelect($("input[name=imageId]"), true);
				return false;
			} else {
				return true;
			}
		}

		self.checkFormValid = function () {
			return $("#mainForm").valid();
		};
		
		self.checkReposPage = function (vm, event) {
			return self.checkFormValid();
		};	

		/**
		 * When the Continue button is clicked, we go from specifications page
		 * to node credentials page.
		 */
		self.continueClicked = function (vm, event) {
			if (location.hash === "" && self.checkSpecsPage(vm, event)){
				location.hash = "creds";
			} else if (location.hash === "#creds" && self.checkCredsPage(vm, event)) {
				location.hash = "repos";
			}
			event.preventDefault();
		};

		self.continueReposClicked = function (vm, event) {
			var result = self.checkReposPage(vm, event);
			if (result) {
				self.submitted(true);
			} else {
				event.preventDefault();
			}
		};

		self.backClicked = function (vm, event) {
			history.go(-1);
		};

		self.checkCredsPage = function (vm, event) {
			resetErrors();
			if (!self.checkFormValid()) {
				return false;
			}
			
			var result = checkEmptyIdentity();
			result = checkEmptyCredential() && result;
			result = checkEmptyPrivateKey() && result;
			result = checkPassphraseMatch() && result;
			return result;
		};
		
		self.checkSpecsPage = function (vm, event) {
			resetErrors();
			if (!self.checkFormValid()) {
				return false;
			}
			
			var result = checkInvalidNumInstances();
			result = checkInvalidCustomGroup() && result;
			result = checkEmptyImageId() && result;
			result = checkEmptyUsername() && result;
			return result;
		};

		/**
		 * When the Start Installation button is clicked.
		 */
		$('#mainForm').submit(function (event) {
			if (self.chosenStep() < 2) {
				self.continueClicked(self, event);
			} else {
				self.continueReposClicked(self, event);
			}
		});

		self.updatePageTitle = function (title) {
			setTimeout(function() {
				$.publish("updatePageTitle", [title]);
			}, 200);
		};
		
		self.goToStart = function () {
			location.hash = "";
		};
		
		self.goBack = function () {
			if (self.chosenStep() === 0) {
				// If we go to the specs page (step 0) by clicking
				// the back button of the the cloud-setup page or by pressing 
				// the browser's refresh button, the browser 
				// history is not how we'd like it, since we had sammy 
				// redirect to the specs page (e.g. simply executing 
				// history.go(-1) from the specs page under these conditions 
				// could lead to the creds page -- step 1 -- being rendered). 
				// Thus, we need to manually go to the provider selection page 
				// (prevPage).
				window.location.href = options.prevPage;
			} else {
				history.go(-1);
			}
		};

		// Reason that we need these regexes as opposed to a simple "#creds" 
		// and "#repos" match is because when you try to go back from 
		// "#creds" (pg. 2) to "" (instance specifications, pg. 1), nothing happens, 
		// since we have no explicit handler for "" (and if we included one, 
		// the quit button would be messed up, since "/cmf/hardware/hosts"
		// [the destination of the quit button], which has no hash, would match 
		// against "").
		$.sammy(function () {
			this.get(/specs\?provider=[A-Za-z0-9_\-]+#repos/, function () {
				// regex matches "specs?provider=<whatever>#repos"
				if (self.initialized) {
					self.chosenStep(2);
					self.updatePageTitle($("#reposTitle").text());
				} else { // page was refreshed, so we lost the content of previous pages
					self.goToStart();
				}
			});
			
			this.get(/specs\?provider=[A-Za-z0-9_\-]+#creds/, function () {
				// regex matches "specs?provider=<whatever>#creds"
				if (self.initialized) {
					self.chosenStep(1);
					self.updatePageTitle($("#credsTitle").text());
				} else { // page was refreshed, so we lost the content of previous pages
					self.goToStart();
				}
			});
			
			// The reason that we don't include the '?' and subsequent 
			// characters in this regex is that if there is no content 
			// after the first '#' in a URL (or if there is no '#' at all), 
			// sammy will just strip away everything 
			// after the first '?' it encouters (assuming that the first '?' comes
			// before the first '#'). So "specs?provider=<whatever>" 
			// (no hash at all) or "specs?provider<whatever>#" (nothing after 
			// the hash) both become just "specs." But 
			// something like "specs?provider=<whatever>#<hash-content>" is 
			// not altered.
			this.get(/specs/, function () {
				self.chosenStep(0);
				self.updatePageTitle($("#specsTitle").text());
			});
		}).run();
		
		self.initialized = true;
		
	};
});
