// Copyright (c) 2011-2012 Cloudera, Inc.  All rights reserved.
define([
	"cloudera/Util",
	"cloudera/common/I18n",
	"cloudera/cmf/include/ProgressBar",
	"cloudera/cmf/include/InstallDetailsDialog",
	"cloudera/Analytics"
], function (Util, I18n, ProgressBar, InstallDetailsDialog, analytics) {

return function(options) {

	var UPDATE_INTERVAL_MS = 1000;
	var ROW_CHILD_SUFFIXES = [
		'hostname',
		'progress',
		'progressSpan',
		'retryLinkContainer',
		'status',
		'statusIcon',
		'statusSpan'
	];

	var progressSum = 0;
	var totalCount = options.configuratorCount;

	var successfulConfiguratorIds = {};
	var failedConfiguratorIds = {};
	var updateTimeout = null;

	function scheduleNextUpdate(){
		updateTimeout = setTimeout(fetchData, UPDATE_INTERVAL_MS);
	}

	function fetchData(){
		$.ajax({
			type: 'POST',
			url: options.installProgressDataURL,
			data: {
				'id': options.requestId
			},
			success: updateProgressTable,
			error: ajaxError,
			dataType: 'json'
		});
	}

	function updateProgressTable(response){
		var configuratorId;

		progressSum = 0;
		for (configuratorId in response.configurators){
			var statusEl = $("#configurator" + configuratorId + "-status");
			var configurator = response.configurators[configuratorId];
			if (configurator.isErrorState) {
				analytics.setCustomVar(2, 'Error', 'Config:' +
						configuratorId +
						',FailedState:' +
						configurator.failedState);
			}
			if (statusEl) {
				// Status element missing. Might be a new (clone) configurator
				// that hasn't been set up in the UI yet, or an abandoned
				// configurator removed from the UI. Skip it.

				if (failedConfiguratorIds[configuratorId] || successfulConfiguratorIds[configuratorId]) {
					progressSum += 100;
				} else {
					var stateRatio = configurator.totalStates === 0 ? 1 :
						configurator.stateNum / configurator.totalStates;
					var status;

					var isDone = configurator.stateNum === configurator.totalStates;
					if (configurator.isErrorState) {
						status = (configurator.state === "ABORT") ? configurator.displayState :
							options.errorOccurred + " " + configurator.displayState;
						if (isDone) {
							$('#configurator' + configuratorId + '-retryLinkContainer').removeClass('hidden');
							failedConfiguratorIds[configuratorId] = true;
						}
					} else {
						status = configurator.displayState;
						if (isDone) {
							successfulConfiguratorIds[configuratorId] = true;
						}
					}
					var progress = 100 * (configurator.totalStates === 0 ? 1 : stateRatio);
					updateStatusDisplay(configuratorId, status, progress, configurator.isErrorState, isDone);
					progressSum += progress;
				}
			}
		}

		var successfulCount = getSuccessfulCount();
		var failedCount = getFailedCount();

		$('#successfulCount').html(successfulCount);
		updateFailedCountDisplay();
		updateOverallProgressDisplay(progressSum);

		var overallProgressBarEl = $('#overallProgress').parent();
		var message;

		if ((successfulCount + failedCount) === totalCount){
			if (failedCount === 0){
				message = "Installation completed successfully.";
				overallProgressBarEl.ProgressBar('success');
			} else if (successfulCount > 0){
				message = "Installation completed successfully on some hosts.";
			} else {
				message = "Installation failed on all hosts.";
				overallProgressBarEl.ProgressBar('fail');
			}
			$('#installHeadline').html(message);

			showContinue();
			updateTimeout = null;
		} else {
			scheduleNextUpdate();
		}
	}

	function keys(o){
		var ret=[],p;
		for(p in o) {
			if (Object.prototype.hasOwnProperty.call(o,p)) {
				ret.push(p);
			}
		}
		return ret;
	}

	function getSuccessfulCount(){
		return keys(successfulConfiguratorIds).length;
	}

	function getFailedCount(){
		return keys(failedConfiguratorIds).length;
	}

	function showAbort(){
		$('#abortButton').removeClass('hidden');
		$('#continueButton').addClass('hidden');
		$('#backQuitButtons').addClass('hidden');
		completeCount = 0;
	}

	function showContinue(){
		$('#abortButton').addClass('hidden');
		// If we have some failed nodes and some successful ones, AND this is 
		// an installation on cloud nodes, we want the continue button 
		// to terminate the failed nodes before it proceeds (so that we can 
		// save the user money).
		if (options.isCloud && getFailedCount() > 0 && getSuccessfulCount() > 0) {
			$('#continueButton').html(options.continueWithFailuresCloud);
		}
		
		$('#continueButton').removeClass('hidden');
		$('#backQuitButtons').removeClass('hidden');
	}

	function abortFormSubmit(event){
		$.ajax({
			type: 'POST',
			url: options.abortURL,
			data: {
				'requestId': options.requestId
			},
			error: ajaxError,
			dataType: 'json'
		});

		$('.InstallStatus').each(function(i, el){
			var configuratorId = getConfiguratorIdFromRowChild(el);
			// skip if configurator already completed (check failed or succeeded)
			if (!(failedConfiguratorIds[configuratorId] || successfulConfiguratorIds[configuratorId])){
				updateStatusDisplay(configuratorId, 'Aborting...', 0, true, false);
			}
		});

		$('#abortButton').addClass('hidden');

		event.preventDefault();
		event.stopPropagation();
	}

	function continueFormSubmit(event){
		if (getSuccessfulCount() === 0) {
			// show incomplete dialog
			$("#installIncompleteDialog").dialog("open");
			$("#continueButton").removeClass("disabled");
			event.preventDefault();
			return false;
		// 'Terminate failed instances and continue' case
		} else if (options.isCloud && getFailedCount() > 0 && getSuccessfulCount() > 0) {
			event.preventDefault();
			cloudTerminate({cloudConfigId: options.cloudConfiguratorId, 
				postTerminate: 'url', postUrl: options.inspectorUrl, 
				requestId: options.requestId, failedNodeConfigIds: 
				String(keys(failedConfiguratorIds))});
		}
	}

	function getConfiguratorIdFromRowChild(el) {
		// remove "configurator" prefix
		return $(el).parents('tr').attr('id').substring(12);
	}

	function updateStatusDisplay(configuratorId, message, progress, errorState, isDone){
		// update status message, icon, and color
		var statusEl = $("#configurator" + configuratorId + "-status");
		var statusIconEl = $("#configurator" + configuratorId + "-statusIcon");
		var statusSpanEl = $("#configurator" + configuratorId + "-statusSpan");
		statusEl.html(message);
		if (isDone) {
			if (errorState) {
				statusIconEl.attr('class', 'IconError16x16');
			} else {
				statusIconEl.attr('class', 'IconCheckmark16x16');
			}
		} else {
			statusIconEl.attr('class', 'IconSpinner16x16');
		}
		if (errorState) {
			statusEl.addClass('error');
		} else {
			statusEl.removeClass('error');
		}

		// update progress bar width and color
		var progressEl = $("#configurator" + configuratorId + "-progress");
		var progressSpan = $("#configurator" + configuratorId + "-progressSpan");
		var roundedProgress = Math.round(progress);
		progressEl.ProgressBar({'percent': roundedProgress});
		if (isDone) {
			progressEl.ProgressBar('success');
		} else {
			progressEl.ProgressBar('reset');
		}

		if (errorState) {
			progressEl.ProgressBar('fail');
		} else {
			progressEl.ProgressBar('reset');
		}
	}

	function showRetryProgress(configuratorId){
		$("#configurator" + configuratorId + "-retryLinkContainer").addClass('hidden');
		updateStatusDisplay(configuratorId, "Retrying...", 0, false, false);
		var progressEl = $("#configurator" + configuratorId + "-progress");
		var progressSpan = $("#configurator" + configuratorId + "-progressSpan");
		progressEl.ProgressBar('reset');
		progressEl.ProgressBar({'percent': 0});
	}

	function sendRetryRequest(configuratorIds){
		$.ajax({
			type: 'POST',
			url: options.retryURL,
			data: {
				requestId: options.requestId,
				configuratorIds: configuratorIds.join(',')
			},
			success: retryCallback,
			error: ajaxError,
			dataType: 'json'
		});

		showAbort();

		// Clear this so we don't retry the same configurators in the future.
		$.each(configuratorIds, function(i, configuratorId){
			delete failedConfiguratorIds[configuratorId];
		});

		$('#installHeadline').html("Installation in progress.");
		var overallProgressBarEl = $('#overallProgress').parent();
		overallProgressBarEl.ProgressBar('reset');
		overallProgressBarEl.ProgressBar({'percent': 0});
		progressSum -= 100;
		updateOverallProgressDisplay();

		if (!updateTimeout){
			scheduleNextUpdate();
		}
	}

	function updateOverallProgressDisplay(){
		var overallProgressBarEl = $("#overallProgress").parent();
		var percent = 100;
		if (totalCount !== 0) {
			percent = Math.round(progressSum / totalCount);
		}
		overallProgressBarEl.ProgressBar({'percent': percent});
	}

	function updateFailedCountDisplay(){
		var failedCount = getFailedCount();
		$('#failedCount').html(failedCount);
		if (failedCount > 0) {
			$('.errorSummary').removeClass('hidden');
		} else {
			$('.errorSummary').addClass('hidden');
		}
	}

	function retryClick(event){
		var configuratorId = getConfiguratorIdFromRowChild(event.target);
		showRetryProgress(configuratorId);
		sendRetryRequest([configuratorId]);

		updateFailedCountDisplay();

		event.preventDefault();
		event.stopPropagation();
	}

	function retryAllClick(event){
		var configuratorId;
		for (configuratorId in failedConfiguratorIds){
			showRetryProgress(configuratorId);
		}

		sendRetryRequest(keys(failedConfiguratorIds));
		updateFailedCountDisplay();

		event.preventDefault();
		event.stopPropagation();
	}

	function retryCallback(response){
		var i, oldId;

		for (oldId in response.configurators){
			var newId = response.configurators[oldId];
			$('#configurator' + oldId).attr('id', 'configurator' + newId);
			for (i = 0; i < ROW_CHILD_SUFFIXES.length; i += 1) {
				var suffix = ROW_CHILD_SUFFIXES[i];
				$('#configurator' + oldId + '-' + suffix).attr('id', 'configurator' + newId + '-' + suffix);
			}
		}
	}

	function ajaxError(response) {
		$.publish("showError", (response.errorMessage || I18n.t("ui.sorryAnError")) 
				+ " " +  I18n.t("ui.tryAgain"));
		hideProgress();
		return false;
	}

	function installDetailsClick(event) {
		var configuratorId = getConfiguratorIdFromRowChild(event.target);

		var url = options.installDetailsURL + options.requestId + '/' + configuratorId;

		installDetailsDialog.show(url);

		event.stopPropagation();
		event.preventDefault();
	}
	
	// Navigate to cloud termination progress page.
	function cloudTerminate(terminateInfo) {
		window.location.href = options.cloudTerminateUrl + '?' + 
			$.param(terminateInfo);
	}
	
	function quitCloudClick() {
		// Load the termination progress page (terminate all started instances); 
		// after termination, navigate to exit URL.
		cloudTerminate({cloudConfigId: options.cloudConfiguratorId, postTerminate: 
			'url', postUrl: options.exitUrl});
	}
	
	function backCloudClick(event) {
		// Load the termination progress page (terminate all started instances);
		// after termination, go back two pages page from this point (from 
		// install_progress). We want to go back two pages rather than one 
		// because we want to bypass cloud-setup, since that page is no longer 
		// relevant. (The nodes whose setup cloud-setup was profiling are now dead.)
		cloudTerminate({cloudConfigId: options.cloudConfiguratorId, postTerminate: 
			'back', stepsBack: 2});
	}

		$('.retryLink').click(retryClick);
		$('.installDetailsLink').click(installDetailsClick);
		$('#retryAllLink').click(retryAllClick);
		$('#abortForm').submit(abortFormSubmit);
		$('#continueForm').submit(continueFormSubmit);
		// If it's not a cloud install, a handler has already been registered 
		// (see JS in AddHostsWizardInstall.jamon).
		if (options.isCloud) {
			$("#quitButton").click(quitCloudClick);
			$("#backButton").click(backCloudClick);
		}

		var dialogOptions = {
			autoOpen: false,
			modal: true,
			resizable: true,
			width: 740,
			maxHeight: 700
		};
		dialogOptions.title = I18n.t("ui.installIncomplete");
		$("#installIncompleteDialog").dialog(dialogOptions);

		scheduleNextUpdate();

		var detailsDialogOptions = {
			detailsDialogSelector: options.detailsDialogSelector
		};
		var installDetailsDialog = new InstallDetailsDialog(detailsDialogOptions);
};
});
